package jp.co.intra_mart.cookbook.im_cookbook_113639.oauth.client.service.processor.impl;

import jp.co.intra_mart.foundation.oauth.client.OAuthClientException;
import jp.co.intra_mart.foundation.oauth.client.model.AccessTokenInfo;
import jp.co.intra_mart.foundation.oauth.client.service.OAuthRequestException;
import jp.co.intra_mart.foundation.oauth.client.service.OAuthResponseParseException;
import jp.co.intra_mart.system.oauth.client.http.ResponseModel;
import jp.co.intra_mart.system.oauth.client.http.entity.HttpEntityModel;
import jp.co.intra_mart.system.oauth.client.message.OAuthClientLog;
import jp.co.intra_mart.system.oauth.client.service.RefreshTokenRequest;
import jp.co.intra_mart.system.oauth.client.service.processor.AbstractRefreshAccessTokenRequestProcessor;

/**
 * アクセストークン更新時の成功レスポンスに refresh_token が含まれない場合に利用する、アクセストークン更新処理クラスです。<br>
 * 本クラスでは、アクセストークン更新時の成功レスポンスに refresh_token が含まれていない場合に、以前のリフレッシュトークンを再利用するように実装しています。<br>
 */
public class RefreshAccessTokenRequestProcessorWithoutRefreshTokenOnSuccessResponse extends AbstractRefreshAccessTokenRequestProcessor {

    private final ThreadLocal<String> previousRefreshTokens = new ThreadLocal<String>();

    protected RefreshAccessTokenRequestProcessorWithoutRefreshTokenOnSuccessResponse(final String providerType) {
        super(providerType);
    }

    @Override
    protected RefreshTokenRequest createRequest(final String providerId, final String refreshToken) throws OAuthClientException {
        previousRefreshTokens.set(refreshToken);
        return super.createRequest(providerId, refreshToken);
    }

    @Override
    protected AccessTokenInfo parseResponse(final String providerId, final ResponseModel<HttpEntityModel<?>> response) throws OAuthClientException, OAuthRequestException {
        try {
            final AccessTokenInfo accessTokenInfo = super.parseResponse(providerId, response);
            final String refreshTokenOnResponse = accessTokenInfo.getRefreshToken();

            // アクセストークン更新時の成功レスポンスに refresh_token が含まれていない場合は、以前のリフレッシュトークンを再利用する。
            if (refreshTokenOnResponse == null || refreshTokenOnResponse.isEmpty()) {
                accessTokenInfo.setRefreshToken(previousRefreshTokens.get());
            }

            return accessTokenInfo;
        } finally {
            previousRefreshTokens.remove();
        }
    }

    @Override
    protected void validateToken(final AccessTokenInfo tokenInfo) throws OAuthResponseParseException {
        try {
            super.validateToken(tokenInfo);
        } catch (final OAuthResponseParseException e) {
            // アクセストークン更新時の成功レスポンスに refresh_token が含まれていない場合、エラーとしない。
            if (e.getMessage().contains(OAuthClientLog.E_IWP_OAUTHCLIENT_PROCESSOR_00016.getWithCode())) {
                return;
            } else {
                throw e;
            }
        }
    }

}
